# This file is part of Tryton.  The COPYRIGHT file at the top level of
# this repository contains the full copyright notices and license terms.
from datetime import timedelta
from trytond.tests.test_tryton import ModuleTestCase, with_transaction
from trytond.pool import Pool
from trytond.transaction import Transaction
from decimal import Decimal
from trytond.modules.company.tests import create_company, set_company
from trytond.modules.employee_timetracking.tests.testlib import create_employee


class EmployeeTestCase(ModuleTestCase):
    'Test employee module'
    module = 'employee_timetracking'

    @with_transaction()
    def test_employee_holidays(self):
        """ test: holidays of employee
        """
        company = create_company('m-ds 1')
        self.assertTrue(company)
        with set_company(company):
            employee = create_employee(company, name='Frida')
            self.assertTrue(employee)
            
            employee.holidays = 25
            employee.specleave = 5
            employee.save()
            self.assertEqual(str(employee.holidays), '25')
            self.assertEqual(str(employee.specleave), '5')

    @with_transaction()
    def test_employee_holidays_not_neg(self):
        """ test: holidays of employee not neg
        """
        company = create_company('m-ds 1')
        self.assertTrue(company)
        with set_company(company):
            employee = create_employee(company, name='Frieda')
            self.assertTrue(employee)
            
            employee.holidays = -25
            self.assertRaises(Exception, employee.save)

    @with_transaction()
    def test_employee_specleave_not_neg(self):
        """ test: specleave of employee not neg
        """
        company = create_company('m-ds 1')
        self.assertTrue(company)
        with set_company(company):
            employee = create_employee(company, name='Frieda')
            self.assertTrue(employee)
            
            employee.specleave = -5
            self.assertRaises(Exception, employee.save)

# end EmployeeTestCase
